#ifndef NODE_EVENT_CALLBACK_H
#define NODE_EVENT_CALLBACK_H

#include <Windows.h>
#include <osg/NodeCallback>
#include <osg/PositionAttitudeTransform>
#include <osg/Node>
#include "NodeInputState.h"

#include<iostream>
#include<algorithm>

using namespace std;

class NodeEventCallback : public osg::NodeCallback 
{
public:
	NodeEventCallback() : rotation(0.0) , pos(0.,0.,0.) { }

	NodeEventCallback::NodeEventCallback(NodeInputState* nis)
		: rotation(0.0) , pos(0.,0.,0.) {
		nodeInputState = nis;
	}

	virtual void operator()(osg::Node* node, osg::NodeVisitor* nv) {
		osg::PositionAttitudeTransform* pat = dynamic_cast<osg::PositionAttitudeTransform*> (node);

		if(node) {
			pos = pat->getPosition();
			if (nodeInputState->moveRequest & MOVE_LEFT) {
				nodeInputState->angle = nodeInputState->angle + 1;
				if(nodeInputState->angle > 360) nodeInputState->angle -= 360;
			}
			if (nodeInputState->moveRequest & MOVE_RIGHT) {
				nodeInputState->angle = nodeInputState->angle - 1;
				if(nodeInputState->angle < 0) nodeInputState->angle += 360;
			}

			double nangle = (osg::PI*(nodeInputState->angle))/180.0;
			double nvel = (double)nodeInputState->velocity;
			if (nodeInputState->moveRequest & MOVE_IN) {
				pos.set(pos.x() + nvel*cos(nangle), pos.y() + nvel*sin(nangle), pos.z());
			}
			if (nodeInputState->moveRequest & MOVE_OUT) {
				pos.set(pos.x() - nvel*cos(nangle), pos.y() - nvel*sin(nangle), pos.z());
			}
			
			if (nodeInputState->moveRequest == MOVE_UP) {
				pos.set(pos.x(), pos.y(), pos.z()+nvel);
			}
			if (nodeInputState->moveRequest == MOVE_DOWN) {
				pos.set(pos.x(), pos.y(), pos.z()-nvel);
			}
			
			pat->setPosition(pos);
			pat->setAttitude(osg::Quat(nangle, osg::Vec3d(0,0,1)));
		}

		traverse(node, nv); 
	}
protected:
	osg::Vec3d pos;
	double rotation;
	NodeInputState* nodeInputState; 
};

#endif